#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['squall',
 'squall.middleware',
 'squall.openapi',
 'squall.routing_',
 'squall.validators']

package_data = \
{'': ['*']}

install_requires = \
['starlette ==0.16.0',
 'orjson >=3.6.4,<4.0.0',
 'apischema >=0.16.2,<1.0.0',
 'isal >=0.11.1,<1.0.0']

extras_require = \
{'all': ['requests >=2.24.0,<3.0.0',
         'jinja2 >=2.11.2,<4.0.0',
         'python-multipart >=0.0.5,<0.0.6',
         'itsdangerous >=1.1.0,<3.0.0',
         'pyyaml >=5.3.1,<6.0.0',
         'orjson >=3.6.4,<4.0.0',
         'email_validator >=1.1.1,<2.0.0',
         'uvicorn[standard] >=0.12.0,<0.16.0'],
 'dev': ['python-jose[cryptography] >=3.3.0,<4.0.0',
         'passlib[bcrypt] >=1.7.2,<2.0.0',
         'autoflake >=1.4.0,<2.0.0',
         'flake8 >=3.8.3,<4.0.0',
         'uvicorn[standard] >=0.12.0,<0.16.0'],
 'doc': ['mkdocs >=1.1.2,<2.0.0',
         'mkdocs-material >=7.1.9,<8.0.0',
         'mdx-include >=1.4.1,<2.0.0',
         'mkdocs-markdownextradata-plugin >=0.1.7,<0.3.0',
         'typer-cli >=0.0.12,<0.0.13',
         'pyyaml >=5.3.1,<6.0.0',
         'pydantic >=1.8.2,<2.0.0'],
 'test': ['pytest >=6.2.4,<7.0.0',
          'pytest-cov >=2.12.0,<4.0.0',
          'pytest-mock >=3.6.1,<4.0.0',
          'mypy ==0.910',
          'flake8 >=3.8.3,<4.0.0',
          'black ==21.9b0',
          'isort >=5.0.6,<6.0.0',
          'requests >=2.24.0,<3.0.0',
          'httpx >=0.14.0,<0.19.0',
          'email_validator >=1.1.1,<2.0.0',
          'sqlalchemy >=1.3.18,<1.5.0',
          'peewee >=3.13.3,<4.0.0',
          'databases[sqlite] >=0.3.2,<0.6.0',
          'python-multipart >=0.0.5,<0.0.6',
          'flask >=1.1.2,<3.0.0',
          'anyio[trio] >=3.2.1,<4.0.0',
          'types-orjson ==3.6.0'],
 "test:python_version<'3.7'": ['types-dataclasses ==0.1.7']}

setup(name='python-squall',
      version='0.2.1',
      description='Squall ASGI REST framework',
      author=None,
      author_email='Stanislav Dubrovskyi <s.dubrovskyi@cleverdec.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.9.0',
     )
